function S0n=distort_sg(S0,sigma,blur,field, Knoise,ncorr,coils)

% DISTORT_SG
%
%  Artifacts and effects over image to simmulate real heart MR
% 
% INPUTS
%    	S0: 	Input image
%	sigma: 	[escalar] variance of noise in each coil
%		[matrix] covariance matrix 
%	blur:	variance of noise in edge blurring
%	field:	noneven illumination (0/1)
%	ncorr   spatial noise correlation. ncorr=window width
%       coils:	number of coils (1,4,8)
%       Knoise: kind of noise
%		1: Rician
%		0: Gaussian
%		2: Multiple coil (no parallel)
%		3: Sense
%		4: GRAPPA

%S0=S0(1:end-1,1:end-1);
[Mx My]=size(S0);

%Blur to simulate Partial Volume Effect--------




if blur>0
	Sbig=periodic_sg(S0);
	window = fspecial('gaussian', 11, blur);
	window=window./sum(window(:));
	Sbig=filter2(window,Sbig);
	S0=Sbig(Mx+(1:Mx),My+(1:My));

end

%Field to simulate non-even light----------
if field==1
	ejex=1:Mx;
	ejey=1:My;
	vX=repmat(ejex',[1,My]);
	vY=repmat(ejey,[Mx,1]);
	vX=pi/2.*vX./max(vX(:));
	vY=pi/2.*vY./max(vY(:));
	Ifield=sin((vY+vX)/2);
	S0=S0.*Ifield;
end

%Noise in the system-----------------------

%Rician: one coil or SENSE
if Knoise==1
  if ncorr==1
	Noise=filter2(ones(3)./3,randn(size(S0)+15))+j.*filter2(ones(3)./3,randn(size(S0)+15));
	Noise=Noise(8:end-8,8:end-8);
  else
	Noise=randn(size(S0))+j.*randn(size(S0));
  end
  S0n=abs(S0+sigma.*Noise);
elseif Knoise==0
  if ncorr==1
	Noise=filter2(ones(3)./3,randn(size(S0)+15));
	Noise=Noise(8:end-8,8:end-8);
  else
	Noise=randn(size(S0));
  end
  S0n=abs(S0+sigma.*Noise);
end

%Multiple coil--------------------

if Knoise>1 
  if coils<2
	error('multiple coils needed');
  end
  ejex=1:Mx;
  ejey=1:My;
  vX=repmat(ejex',[1,My]);
  vY=repmat(ejey,[Mx,1]);
  vX=pi/2.*vX./max(vX(:));
  vY=pi/2.*vY./max(vY(:));
  I=S0;

  if coils ==4 %Simulación de Sensitivity field para 4 coils
	It(:,:,1)=I.*cos(vX)./sqrt(2);
	It(:,:,3)=I.*sin(vX)./sqrt(2);
	It(:,:,2)=I.*cos(vY)./sqrt(2);
	It(:,:,4)=I.*sin(vY)./sqrt(2);	
	%Sensitivity map
	MapW(:,:,1)=cos(vX)./sqrt(2);
	MapW(:,:,3)=sin(vX)./sqrt(2);
	MapW(:,:,2)=cos(vY)./sqrt(2);
	MapW(:,:,4)=sin(vY)./sqrt(2);
  elseif coils==8 %Simulación de Sensitivity field para 8 coils

	I1=I.*cos(vX)./2;
	I2=I.*sin(vX)./2;
	I3=I.*cos(vY)./2;
	I4=I.*sin(vY)./2;

	I5=I.*(cos((vY+vX)/2))./2;
	I6=I.*(sin((vY+vX)/2))./2;
	I7=I.*(cos(((pi/2-vY)+vX)/2))./2;
	I8=I.*(sin(((pi/2-vY)+vX)/2))./2;
	%reordenación
	It(:,:,1)=I1;
	It(:,:,2)=I5;
	It(:,:,3)=I3;
	It(:,:,4)=I8;

	It(:,:,5)=I2;
	It(:,:,6)=I6;
	It(:,:,7)=I4;
	It(:,:,8)=I7;

	%Sensitivity map
	MapW(:,:,1)=cos(vX)./sqrt(2);
	MapW(:,:,5)=sin(vX)./sqrt(2);
	MapW(:,:,3)=cos(vY)./sqrt(2);
	MapW(:,:,7)=sin(vY)./sqrt(2);
	MapW(:,:,2)=(cos((vY+vX)/2))./2;
	MapW(:,:,6)=(sin((vY+vX)/2))./2;
	MapW(:,:,8)=(cos(((pi/2-vY)+vX)/2))./2;
	MapW(:,:,4)=(sin(((pi/2-vY)+vX)/2))./2;
   end


if length(sigma)==1
	%No coil correlation
	if ncorr==0
	  Nc2=sigma.*(randn(size(It))+j.*randn(size(It)));
	else
	[Nx,Ny,Nz]=size(It);
	  for ii=1:Nz
	  Nc2(:,:,Nz)=filter2(ones(3)./3,randn(size(S0)+15))+j.*filter2(ones(3)./3,randn(size(S0)+15));
	  end
	  Nc2=sigma.*Nc2(8:end-8,8:end-8,:);
	end
else
	%Covariance matrix considered
        if ncorr==0
	
	Nc=randn([Mx,My,coils])+j.*randn([Mx,My,coils]);	
	else %ncorr
        
	  for ii=1:coils
	  Nc(:,:,ii)=filter2(ones(3)./3,randn(size(S0)+15))+j.*filter2(ones(3)./3,randn(size(S0)+15));
	  end
	  Nc=Nc(8:end-8,8:end-8,:);

	end %Ncorr

   	[V,D] = eig(sigma);
   	W = V*sqrt(D);
   	for ii=1:Mx
   	for jj=1:My
		Nc2(ii,jj,:)=W*(squeeze(Nc(ii,jj,:)));
   	end
   	end

	%Int=It+Nc2;
end

Int=It+Nc2;

if Knoise==2
  S0n=sos(Int);
elseif Knoise==3 %SENSE

error('SENSE not implemented in this version');
  %k=x2k(Int); 	%K-space
  %TasaM=2; 	%Acceleration rate
  %SENSE reconstruction
  %Ics= sense2(permute(MapW,[2,1,3]), permute(k( 1:TasaM:end,:,:),[2,1,3]), TasaM)';
  %S0n=abs(Ics);   %Señal compuesta reconstruída con SENSE
elseif Knoise==4 %GRAPPA
  %k=x2k(Int);
  %[Nx Ny Lz] = size(k);
  %Subsampling 2x
  %Z = zeros(Nx,1);
  %Z( 2:2:end) = 1;  	
  %Z( (Nx/2) + (-round(Nx/8):round(Nx/8)) ) = 1;	%16 ACS lines 
  %Y = find(Z);
  %k_m1=zeros(size(k));
  %k_m1(Y,:,:)=k(Y,:,:); 

  %GRAPPA Reconstruction
  %[Fc,Ic,Nc]=recongrappa_multik([Nx Ny Lz],k_m1(Y,:,:),Y ,'dks',2);

  %S0n=sos(k2x(Fc,1)); 
error('GRAPPA not implemented in this version');
end

end %if coils>1
